// CoMET - The Crimson Fields Map Editing Tool
// Copyright (C) 2002-2003 Jens Granseuer
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

////////////////////////////////////////////////////////////////////////
// extwindow2.cpp
//
// History:
//  06-12-2002 - created
///////////////////////////////////////////////////////////////////////

#include <stdlib.h>

#include "extwindow2.h"
#include "extwindow.h"
#include "aux.h"

// local static vars
static const char *player2_labels[] = { "One", "Two", 0 };
static const char *player3_labels[] = { "One", "Two", "Neutral", 0 };
static const char *dir_labels[] = { "North", "Northeast", "Southeast",
                                    "South", "Southwest", "Northwest", 0 };
extern const char *event_labels[];
extern const char *etrigger_labels[];


////////////////////////////////////////////////////////////////////////
// NAME       : NewMissionWindow::NewMissionWindow
// DESCRIPTION: This window pops up when the user wants to create a new
//              mission. He can adjust a few basic settings like map
//              size or tileset and create the mission.
// PARAMETERS : view - pointer to the window's view
// RETURNS    : -
//
// HISTORY
////////////////////////////////////////////////////////////////////////

NewMissionWindow::NewMissionWindow( View *view ) : Window(WIN_CENTER, view) {

  // calculate window dimensions
  Font *f = SmallFont();
  unsigned short w = MIN( f->Width() * 35 + 54, view->Width() - 20 );
  unsigned short h = f->Height() * 3 + 40;
  unsigned short xoff = f->Width() * 11 + 14, yoff = 5;

  SetSize( w, h );

  m_width = new NumberWidget( 1, xoff, yoff, f->Width() * 4 + 10,
            f->Height() + 6, 'w', 40, 20, 200, WIDGET_ALIGN_LEFT,
            "Map Width :", this );

  tileset = new StringWidget( 3, xoff + m_width->w + 10 + f->Width() * 9,
            yoff, f->Width() * 11 + 10, m_width->h, 't', "default", 10,
            WIDGET_DISABLED|WIDGET_ALIGN_LEFT, "Tile Set:", this );

  yoff += m_width->Height() + 5;

  m_height = new NumberWidget( 2, xoff, yoff, m_width->w, m_width->h,
            'h', 40, 20, 200, WIDGET_ALIGN_LEFT, "Map Height:", this );

  unitset = new StringWidget( 4, tileset->x, yoff, tileset->w, tileset->h,
            'u', "default", 10, WIDGET_DISABLED|WIDGET_ALIGN_LEFT,
            "Unit Set:", this );

  yoff = h - f->Height() - 15;
  ok = new ButtonWidget( B_ID_NMW_CREATE, 10, yoff, f->Width() * 10,
       f->Height() + 10, 'o', 0, "OK", this );

  new ButtonWidget( GUI_CLOSE, w - 10 - ok->w, yoff, ok->w, ok->h,
                    'c', 0, "Cancel", this );
  Draw();
  Show();
}

////////////////////////////////////////////////////////////////////////
// NAME       : NewMissionWindow::GetMapSize
// DESCRIPTION: Get the current slider settings.
// PARAMETERS : -
// RETURNS    : selected size as a Point
//
// HISTORY
////////////////////////////////////////////////////////////////////////

Point NewMissionWindow::GetMapSize( void ) const {
  Point s( 0, 0 );

  s.x = m_width->Number();
  s.y = m_height->Number();

  return s;
}


////////////////////////////////////////////////////////////////////////
// NAME       : EdBuildingWindow::EdBuildingWindow
// DESCRIPTION: This window contains the settings for a building.
// PARAMETERS : bld   - building to be modified
//              first - first unit currently on the map
//              view  - pointer to the window's view
// RETURNS    : -
//
// HISTORY
////////////////////////////////////////////////////////////////////////

EdBuildingWindow::EdBuildingWindow( Building &bld, Unit *first, View *view ) :
                  Window(WIN_CENTER, view), b(bld), unit(first) { 
  // calculate dimensions
  SetSize( MIN(sfont->Width() * 50, view->Width()),
           MIN(sfont->Height() * 12, view->Height()) );

  unsigned short xoff, yoff = 5, wdh = sfont->Height() + 10;

  b_name = new StringWidget( B_ID_NAME, 5, yoff, w/2 - 10, wdh,
                             'n', b.Name(), 19, 0, NULL, this );

  xoff = w/2 + 10 + sfont->TextWidth("Player");
  b_player = new CycleWidget( B_ID_PLAYER, xoff, yoff, w - 5 - xoff, wdh,
                              'p', 0, "Player", b.Owner(), player3_labels, this );
  yoff += 5 + wdh;

  ButtonWidget *btn;
  btn = new CheckboxWidget( B_ID_WORKSHOP, 5, yoff, DEFAULT_CBW_SIZE,
                            DEFAULT_CBW_SIZE, b.IsWorkshop(), 'w',
                            WIDGET_STYLE_NOBORDER|WIDGET_STYLE_GFX|WIDGET_ALIGN_RIGHT,
                            "Workshop", this );
  btn->SetHook( this );

  btn = new CheckboxWidget( B_ID_FACTORY, 5, yoff + DEFAULT_CBW_SIZE + 5,
                            DEFAULT_CBW_SIZE, DEFAULT_CBW_SIZE, b.IsFactory(), 'f',
                            WIDGET_STYLE_NOBORDER|WIDGET_STYLE_GFX|WIDGET_ALIGN_RIGHT,
                            "Factory", this );
  btn->SetHook( this );


  btn = new ButtonWidget( B_ID_OK, 1, h - 1 - wdh, w - 2, wdh, 'o',
                          WIDGET_DEFAULT, "OK", this ),
  btn->SetHook( this );

  Draw();
  Show();
}

////////////////////////////////////////////////////////////////////////
// NAME       : EdBuildingWindow::Activate
// DESCRIPTION: React to user actions.
// PARAMETERS : button - activated button widget
//              win    - window containing the button
// RETURNS    : GUI status
//
// HISTORY
////////////////////////////////////////////////////////////////////////

GUI_Status EdBuildingWindow::Activate( ButtonWidget *button, Window *win ) {

  switch ( button->ID() ) {
  case B_ID_WORKSHOP:
    if ( button->Clicked() ) b.SetFlags( BLD_WORKSHOP );
    else b.UnsetFlags( BLD_WORKSHOP );
    break;
  case B_ID_FACTORY:
    if ( button->Clicked() ) b.SetFlags( BLD_FACTORY );
    else b.UnsetFlags( BLD_FACTORY );
    break;
  case B_ID_OK:
    view->CloseWindow( win );
    b.SetOwner( b_player->GetValue() );
    b.SetName( b_name->String() );

    while ( unit ) {
      if ( unit->Position() == b.Position() ) unit->SetOwner( b.Owner() );
      unit = static_cast<Unit *>(unit->Next());
    }
    break;
  }

  return GUI_OK;
}

////////////////////////////////////////////////////////////////////////
// NAME       : EdUnitWindow::EdUnitWindow
// DESCRIPTION: This window contains the settings for a unit.
// PARAMETERS : unit - unit to be modified
//              view - pointer to the window's view
// RETURNS    : -
//
// HISTORY
////////////////////////////////////////////////////////////////////////

EdUnitWindow::EdUnitWindow( Unit &unit, View *view ) :
              Window(WIN_CENTER, view), u(unit) { 
  // calculate dimensions
  SetSize( MIN(sfont->Width() * 50, view->Width()),
           MIN(sfont->Height() * 12, view->Height()) );

  unsigned short xoff, yoff = 5, wdh = sfont->Height() + 10;

  Widget *wd = new StringWidget( B_ID_NAME, 5, yoff,
                   w/2 - 10, wdh, 0, u.Name(), 19,
                   WIDGET_STYLE_NOBORDER|WIDGET_STR_CONST,
                   NULL, this );
  xoff = w/2 + 10 + sfont->TextWidth("Player");
  u_player = new CycleWidget( B_ID_PLAYER, xoff, yoff, w - 5 - xoff, wdh,
                              'p', 0, "Player", u.Owner(), player2_labels, this );
  u_player->SetHook( this );
  yoff += 5 + wdh;

  new NumberWidget( B_ID_ID,
                    wd->LeftEdge() + 10 + sfont->Width() * 2,
                    yoff, sfont->Width() * 3, wdh, 0, u.ID(), 0, 32000,
                    WIDGET_ALIGN_LEFT|WIDGET_STYLE_NOBORDER|WIDGET_STR_CONST,
                    "ID", this );

  xoff = w/2 + 10 + sfont->TextWidth("Direction");
  ButtonWidget *btn = new CycleWidget( B_ID_DIRECTION,
                          xoff, yoff, w - 5 - xoff, wdh,
                          'd', 0, "Direction", u.GetDirection(), dir_labels, this );
  btn->SetHook( this );
  yoff += 5 + wdh;


  btn = new ButtonWidget( B_ID_OK, 1, h - 1 - wdh, w - 2, wdh,
                          'o', WIDGET_DEFAULT, "OK", this ),
  btn->SetHook( this );

  Draw();
  Show();
}

////////////////////////////////////////////////////////////////////////
// NAME       : EdUnitWindow::Activate
// DESCRIPTION: React to user actions.
// PARAMETERS : button - activated button widget
//              win    - window containing the button
// RETURNS    : GUI status
//
// HISTORY
////////////////////////////////////////////////////////////////////////

GUI_Status EdUnitWindow::Activate( ButtonWidget *button, Window *win ) {
  GUI_Status rc = GUI_OK;

  switch ( button->ID() ) {
  case B_ID_DIRECTION:
    u.SetDirection( static_cast<CycleWidget *>(button)->GetValue() );
    break;
  case B_ID_OK:
    view->CloseWindow( win );
    u.SetOwner( u_player->GetValue() );

    if ( u.IsTransport() && !u.IsSheltered() ) {
      Unit *walk = &u;
      while ( walk->Prev() ) walk = static_cast<Unit *>(walk->Prev());
      while ( walk ) {
        if ( walk->Position() == u.Position() ) walk->SetOwner( u.Owner() );
        walk = static_cast<Unit *>(walk->Next());
      }
    }
    rc = (GUI_Status)0;
    break;
  }

  return rc;
}

////////////////////////////////////////////////////////////////////////
// NAME       : EdEventsWindow::EdEventsWindow
// DESCRIPTION: This window presents the list of events for selection.
// PARAMETERS : mission - current mission
//              view    - pointer to the window's view
// RETURNS    : -
//
// HISTORY
////////////////////////////////////////////////////////////////////////

EdEventsWindow::EdEventsWindow( Mission &mission, View *view ) :
              Window(WIN_CENTER, view), mission(mission) { 
  // calculate dimensions
  SetSize( MIN(sfont->Width() * 20 + 20, view->Width()),
           MIN( MAX(mission.GetEvents().CountNodes() + 2, 6) *
                   (sfont->Height() + 2) + 40,
                view->Height()) );

  TLWListBuilder::BuildEventList( mission.GetEvents(), nodes );

  e_list = new TextListWidget( 0, 5, 5, w - 10, h - sfont->Height() * 2 - 30,
               &nodes, -1, WIDGET_ARROWSCROLL, NULL, this );

  ButtonWidget *btn = new ButtonWidget( B_ID_NEW,
               1, h - sfont->Height() * 2 - 17, (w - 2)/2, sfont->Height() + 8,
               'n', 0, "New", this );
  btn->SetHook( this );

  btn = new ButtonWidget( B_ID_EDIT,
              btn->LeftEdge() + btn->Width(), btn->TopEdge(),
              btn->Width(), btn->Height(), 'e', 0, "Edit", this );
  btn->SetHook( this );

  btn = new ButtonWidget( B_ID_DELETE,
              btn->LeftEdge() - btn->Width(), btn->TopEdge() + btn->Height(),
              btn->Width(), btn->Height(), 'd', 0, "Delete", this );
  btn->SetHook( this );

  btn = new ButtonWidget( B_ID_OK,
              btn->LeftEdge() + btn->Width(), btn->TopEdge(),
              btn->Width(), btn->Height(), 'o', 0, "OK", this );
  btn->SetHook( this );

  Draw();
  Show();
}

////////////////////////////////////////////////////////////////////////
// NAME       : EdEventsWindow::Activate
// DESCRIPTION: React to user actions.
// PARAMETERS : button - activated button widget
//              win    - window containing the button
// RETURNS    : GUI status
//
// HISTORY
////////////////////////////////////////////////////////////////////////

GUI_Status EdEventsWindow::Activate( ButtonWidget *button, Window *win ) {

  switch ( button->ID() ) {
  case B_ID_EDIT:
    new NoteWindow("Sorry", "Not yet implemented", 0, view); // FIXME
    break;
  case B_ID_NEW:
    view->CloseWindow( win );
    new SelectEventWindow( mission, view );
    break;
  case B_ID_DELETE: {
    TLWNode *n = static_cast<TLWNode *>( e_list->Selected() );
    if ( n ) {
      Event *e = (Event *)n->UserData();

      n->Remove();
      delete n;
      e_list->Update();
      e_list->Draw();
      e_list->Show();

      mission.DeleteEvent( e );
    }
    break; }
  case B_ID_OK:
    view->CloseWindow( win );
    break;
  }

  return GUI_OK;
}


////////////////////////////////////////////////////////////////////////
// NAME       : SelectEventWindow::SelectEventWindow
// DESCRIPTION: This window presents the list of event types.
// PARAMETERS : mission - current mission
//              view    - pointer to the window's view
// RETURNS    : -
//
// HISTORY
////////////////////////////////////////////////////////////////////////

SelectEventWindow::SelectEventWindow( Mission &mission, View *view ) :
                   Window(WIN_CENTER, view), mission(mission) { 
  // calculate dimensions
  SetSize( MIN(sfont->Width() * 18 + 20, view->Width()),
           MIN( 10 * (sfont->Height() + 2) + 42, view->Height()) );

  BuildTLWList( nodes );

  e_list = new TextListWidget( 0, 5, 5, w - 10, h - sfont->Height() * 3 - 36,
               &nodes, -1, WIDGET_ARROWSCROLL, NULL, this );

  e_trigger = new CycleWidget( B_ID_TRIGGER, e_list->LeftEdge(),
                  e_list->TopEdge() + e_list->Height() + sfont->Height() + 10,
                  e_list->Width(), sfont->Height() + 8, 't', WIDGET_ALIGN_TOP,
                  "Triggered by", 0, etrigger_labels, this );

  ButtonWidget *btn = new ButtonWidget( B_ID_OK,
               1, h - sfont->Height() - 9, (w - 2)/2, sfont->Height() + 8,
               'o', WIDGET_DEFAULT, "OK", this );
  btn->SetHook( this );

  btn = new ButtonWidget( B_ID_CANCEL,
              btn->LeftEdge() + btn->Width(), btn->TopEdge(),
              btn->Width(), btn->Height(), 'c', 0, "Cancel", this );
  btn->SetHook( this );

  Draw();
  Show();
}

////////////////////////////////////////////////////////////////////////
// NAME       : SelectEventWindow::Activate
// DESCRIPTION: React to user actions.
// PARAMETERS : button - activated button widget
//              win    - window containing the button
// RETURNS    : GUI status
//
// HISTORY
////////////////////////////////////////////////////////////////////////

GUI_Status SelectEventWindow::Activate( ButtonWidget *button, Window *win ) {

  switch ( button->ID() ) {
  case B_ID_OK:

/*** FIXME
    if ( e_list->Selected() ) {
      view->CloseWindow( win );

      Event *e = mission.CreateEvent(
                     static_cast<TLWNode *>(e_list->Selected())->ID(),
                     e_trigger->GetValue() );

      switch ( e->Type() ) {
      case EVENT_CREATE_UNIT:      new EventCreateUnitWindow( e ); break;
      case EVENT_MANIPULATE_EVENT: new EventManipulateEventWindow( e ); break;
      case EVENT_MESSAGE:          new EventMessageWindow( e ); break;
      case EVENT_MINING:           new EventMiningWindow( e ); break;
      case EVENT_NEXT_MAP:         new EventNextMapWindow( e ); break;
      case EVENT_RESEARCH:         new EventResearchWindow( e ); break;
      case EVENT_SCORE:            new EventScoreWindow( e ); break;
      }
    }
***/

    new NoteWindow("Sorry", "Not yet implemented", 0, view);
    break;
  case B_ID_CANCEL:
    view->CloseWindow( win );
    break;
  }

  return GUI_OK;
}

////////////////////////////////////////////////////////////////////////
// NAME       : SelectEventWindow::BuildTLWList
// DESCRIPTION: Create a list for a TextListWidget containing all
//              available event types.
// PARAMETERS : list - list of TLWNodes to be filled
// RETURNS    : -
//
// HISTORY
////////////////////////////////////////////////////////////////////////

void SelectEventWindow::BuildTLWList( TLWList &events ) const {
  for ( int i = 0; event_labels[i]; ++i )
    events.InsertNodeSorted( new TLWNode(event_labels[i], 0, i+1) );
}

