// Crimson Fields -- a game of tactical warfare
// Copyright (C) 2000-2003 Jens Granseuer
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

////////////////////////////////////////////////////////////////////////
// sound.cpp
//
// History:
//  22-04-2001 - created
//  02-06-2001 - sound can be turned off using toggle_audio()
//  01-08-2001 - almost entirely rewritten
//  16-03-2002 - rewritten to use SDL_mixer
//  25-08-2002 - init/free sound subsystem when audio setting toggled
////////////////////////////////////////////////////////////////////////


#include <string>
#include "SDL.h"

#include "sound.h"
#include "fileio.h"

#ifndef DISABLE_SOUND
# define NUM_SFX   6       // system effects; they have nothing to do with
                           // the sounds defined by a mission set

 static bool audio_init = false;
 static SoundEffect *sfx[NUM_SFX];
 static const char *sfx_files[] = { "error.wav", "clicked.wav", "menu.wav",
                                    "ask.wav", "select.wav", "repair.wav" };
#endif

static bool audio_on;

////////////////////////////////////////////////////////////////////////
// NAME       : init_audio
// DESCRIPTION: Initialize the necessary audio structures and load the
//              sound effects.
// PARAMETERS : state - initial state of sound effects (on/off)
// RETURNS    : 0 on complete success, -1 on error
//
// HISTORY
////////////////////////////////////////////////////////////////////////

int init_audio( bool state ) {
  audio_on = state;

#ifndef DISABLE_SOUND
  if ( audio_on ) {
    if ( SDL_InitSubSystem( SDL_INIT_AUDIO ) < 0 ) {
      fprintf( stderr, "Couldn't initialize audio: %s\n", SDL_GetError() );
    } else {
      if ( Mix_OpenAudio( 22050, AUDIO_S16, 2, 1024 ) < 0 ) return -1;
      audio_init = true;

      string sndd( get_sounds_dir() );
      for ( int i = 0; i < NUM_SFX; ++i ) {
        sfx[i] = new SoundEffect( (sndd + sfx_files[i]).c_str() );
      }
    }
  }
#endif

  return 0;
}

////////////////////////////////////////////////////////////////////////
// NAME       : get_audio
// DESCRIPTION: Get the current audio setting (on/off).
// PARAMETERS : -
// RETURNS    : current setting
//
// HISTORY
////////////////////////////////////////////////////////////////////////

bool get_audio( void ) {
  return audio_on;
}

////////////////////////////////////////////////////////////////////////
// NAME       : toggle_audio
// DESCRIPTION: Toggle sound effects on/off.
// PARAMETERS : -
// RETURNS    : new setting
//
// HISTORY
////////////////////////////////////////////////////////////////////////

bool toggle_audio( void ) {
  audio_on ^= 1;

#ifndef DISABLE_SOUND
  if ( audio_on ) init_audio( true );
  else if ( audio_init ) shutdown_audio();
#endif

  return audio_on;
}

////////////////////////////////////////////////////////////////////////
// NAME       : play_audio
// DESCRIPTION: Play a sound effect by its identifier.
// PARAMETERS : sfxid - sound effect identifier
//              flags - see sound.h for details
// RETURNS    : -
//
// HISTORY
////////////////////////////////////////////////////////////////////////

void play_audio( unsigned short sfxid, unsigned short flags ) {
#ifndef DISABLE_SOUND
  if ( (sfxid < NUM_SFX) && sfx[sfxid] ) sfx[sfxid]->Play( flags );
#endif
}


////////////////////////////////////////////////////////////////////////
// NAME       : shutdown_audio
// DESCRIPTION: Shutdown the audio system.
// PARAMETERS : -
// RETURNS    : -
//
// HISTORY
////////////////////////////////////////////////////////////////////////

void shutdown_audio( void ) {
#ifndef DISABLE_SOUND
  if ( audio_init ) {
    Mix_CloseAudio();
    SDL_QuitSubSystem( SDL_INIT_AUDIO );

    for ( int i = 0; i < NUM_SFX; ++i ) {
      delete sfx[i];
      sfx[i] = NULL;
    }
    audio_init = false;
  }
#endif
}


////////////////////////////////////////////////////////////////////////
// NAME       : SoundEffect::SoundEffect
// DESCRIPTION: Load a sound effect from a WAVE file.
// PARAMETERS : file - name of the WAVE file to load
//              dev  - audio spec of the (already open) audio device
// RETURNS    : -
//
// HISTORY
//   04-08-2001 - skip conversion if sound already has required format
////////////////////////////////////////////////////////////////////////

SoundEffect::SoundEffect( const char *file ) {

#ifndef DISABLE_SOUND
  channel = -1;
  if ( !(sample = Mix_LoadWAV( file )) ) {
    fprintf( stderr, "Failed to load sound: %s\n", SDL_GetError() );
  }
#endif

}

////////////////////////////////////////////////////////////////////////
// NAME       : SoundEffect::~SoundEffect
// DESCRIPTION: Free the audio data.
// PARAMETERS : -
// RETURNS    : -
//
// HISTORY
////////////////////////////////////////////////////////////////////////

SoundEffect::~SoundEffect( void ) {
#ifndef DISABLE_SOUND
  if ( sample ) Mix_FreeChunk( sample );
#endif
}

////////////////////////////////////////////////////////////////////////
// NAME       : SoundEffect::Play
// DESCRIPTION: Play a sound effect.
// PARAMETERS : flags - see sound.h for details
// RETURNS    : -
//
// HISTORY
////////////////////////////////////////////////////////////////////////

void SoundEffect::Play( unsigned short flags ) {

#ifndef DISABLE_SOUND
  if ( audio_init && audio_on && sample ) {
    channel = Mix_PlayChannel( -1, sample, (flags & SFX_LOOP ? -1 : 0) );
  }
#endif

}

////////////////////////////////////////////////////////////////////////
// NAME       : SoundEffect::Stop
// DESCRIPTION: Stop playing a sound effect.
// PARAMETERS : -
// RETURNS    : -
//
// HISTORY
////////////////////////////////////////////////////////////////////////

void SoundEffect::Stop( void ) {
#ifndef DISABLE_SOUND
  if ( channel >= 0 ) {
    if ( Mix_Playing( channel ) ) Mix_HaltChannel( channel );
    channel = -1;
  }
#endif
}

