/* mkdatafile -- create a Crimson Fields data file
   Copyright (C) 2000-2002 Jens Granseuer

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/* takes five file names: icons BMP, small font BMP, large font BMP, and output filename

   The data file is written in little-endian format.

   Creates a data file for Crimson Fields containing fonts and icons image data
*/

#include <stdio.h>
#include <stdlib.h>

#ifdef HAVE_LIBZ
# include <zlib.h>
#endif

#include "SDL.h"
#include "SDL_endian.h"

static int compress_data( unsigned char *in, unsigned long inlen, unsigned char **out, unsigned long *outlen );

#define FONT_CAPS_ONLY	0x01

struct FontInfo {
  unsigned char w;
  unsigned char h;
  unsigned char chars;
  unsigned char flags;
};

struct PicHeader {
  unsigned short colors;
  unsigned short width;
  unsigned short height;
  unsigned long packed_size;
};


int main( int argc, char *argv[] ) {
  SDL_Surface *bmp;
  int status, i, j, size;
  SDL_RWops *out;
  unsigned char *packbuf, *upackbuf;
 
  if ( argc != 5 ) {
    fprintf(stderr, "Invalid number of arguments\n"
                    "Usage: %s <icons.bmp> <smallfont.bmp> <largefont.bmp> <outfile>\n",
                    argv[0] );
    exit(-1);
  }


  if ( SDL_Init(0) < 0 ) {
    fprintf(stderr, "Couldn't init SDL: %s\n", SDL_GetError());
    exit(-1);
  }
  atexit(SDL_Quit);

  out = SDL_RWFromFile( argv[4], "wb" );
  if ( !out ) {
    fprintf(stderr, "Couldn't open output file %s\n", argv[5] );
    exit(-1);
  }

  status = 0;

  /* icons */
  bmp = SDL_LoadBMP(argv[1]);
  if ( bmp ) {
    if ( bmp->format->BitsPerPixel == 8 ) {
      SDL_Palette *palette = bmp->format->palette;
      struct PicHeader pict = { palette->ncolors, bmp->w, bmp->h, 0 };

      size = bmp->w * bmp->h;
      upackbuf = (unsigned char *)malloc( size );

      /* copy surface lines to buffer */
      for ( i = 0; i < bmp->h; ++i ) {
        unsigned char *pix = (Uint8 *)bmp->pixels + i * bmp->pitch;
        for ( j = 0; j < bmp->w; ++j ) {
          upackbuf[j + i * bmp->w] = *pix++;
        }
      }

      status = compress_data( upackbuf, size, &packbuf, &pict.packed_size );
      if ( status ) {
        fprintf( stderr, "Error compressing data\n" );
        free( upackbuf );
        SDL_RWclose( out );
        return status;
      }

      SDL_WriteLE16( out, pict.colors );
      SDL_WriteLE16( out, pict.width );
      SDL_WriteLE16( out, pict.height );
      SDL_WriteLE32( out, pict.packed_size );

      for ( i=0; i<palette->ncolors; ++i ) {
        SDL_RWwrite( out, &palette->colors[i].r, 1, 1 );
        SDL_RWwrite( out, &palette->colors[i].g, 1, 1 );
        SDL_RWwrite( out, &palette->colors[i].b, 1, 1 );
      }

      SDL_RWwrite( out, packbuf, 1, pict.packed_size );
      if ( pict.packed_size != size ) free( packbuf );
    } else {
      fprintf(stderr, "Couldn't find palette\n" );
      status = -1;
    }
    SDL_FreeSurface( bmp );
  } else {
    fprintf(stderr, "Couldn't open BMP %s\n", argv[2] );
    status = -1;
  }

  /* small font */
  bmp = SDL_LoadBMP(argv[2]);
  if ( bmp ) {
    if ( bmp->format->BitsPerPixel == 8 ) {
      SDL_Palette *palette = bmp->format->palette;
      struct PicHeader pict = { palette->ncolors, bmp->w, bmp->h, 0 };
      struct FontInfo font = { 7, 12, 90, 0 };

      size = bmp->w * bmp->h;
      upackbuf = (unsigned char *)malloc( size );

      /* copy surface lines to buffer */
      for ( i = 0; i < bmp->h; ++i ) {
        unsigned char *pix = (Uint8 *)bmp->pixels + i * bmp->pitch;
        for ( j = 0; j < bmp->w; ++j ) {
          upackbuf[j + i * bmp->w] = *pix++;
        }
      }

      status = compress_data( upackbuf, size, &packbuf, &pict.packed_size );
      if ( status ) {
        fprintf( stderr, "Error compressing data\n" );
        free( upackbuf );
        SDL_RWclose( out );
        return status;
      }

      SDL_RWwrite( out, (char *)&font, sizeof(struct FontInfo), 1 );

      SDL_WriteLE16( out, pict.colors );
      SDL_WriteLE16( out, pict.width );
      SDL_WriteLE16( out, pict.height );
      SDL_WriteLE32( out, pict.packed_size );

      for ( i=0; i<palette->ncolors; ++i ) {
        SDL_RWwrite( out, &palette->colors[i].r, 1, 1 );
        SDL_RWwrite( out, &palette->colors[i].g, 1, 1 );
        SDL_RWwrite( out, &palette->colors[i].b, 1, 1 );
      }

      SDL_RWwrite( out, packbuf, 1, pict.packed_size );
      if ( pict.packed_size != size ) free( packbuf );
    } else {
      fprintf(stderr, "Couldn't find palette\n" );
      status = -1;
    }
    SDL_FreeSurface( bmp );
  } else {
    fprintf(stderr, "Couldn't open BMP %s\n", argv[3] );
    status = -1;
  }

  /* large font */
  bmp = SDL_LoadBMP(argv[3]);
  if ( bmp ) {
    if ( bmp->format->BitsPerPixel == 8 ) {
      SDL_Palette *palette = bmp->format->palette;
      struct PicHeader pict = { palette->ncolors, bmp->w, bmp->h, 0 };
      struct FontInfo font = { 12, 14, 64, FONT_CAPS_ONLY };

      size = bmp->w * bmp->h;
      upackbuf = (unsigned char *)malloc( size );

      /* copy surface lines to buffer */
      for ( i = 0; i < bmp->h; ++i ) {
        unsigned char *pix = (Uint8 *)bmp->pixels + i * bmp->pitch;
        for ( j = 0; j < bmp->w; ++j ) {
          upackbuf[j + i * bmp->w] = *pix++;
        }
      }

      status = compress_data( upackbuf, size, &packbuf, &pict.packed_size );
      if ( status ) {
        fprintf( stderr, "Error compressing data\n" );
        free( upackbuf );
        SDL_RWclose( out );
        return status;
      }

      SDL_RWwrite( out, (char *)&font, sizeof(struct FontInfo), 1 );

      SDL_WriteLE16( out, pict.colors );
      SDL_WriteLE16( out, pict.width );
      SDL_WriteLE16( out, pict.height );
      SDL_WriteLE32( out, pict.packed_size );

      for ( i=0; i<palette->ncolors; ++i ) {
        SDL_RWwrite( out, &palette->colors[i].r, 1, 1 );
        SDL_RWwrite( out, &palette->colors[i].g, 1, 1 );
        SDL_RWwrite( out, &palette->colors[i].b, 1, 1 );
      }

      SDL_RWwrite( out, packbuf, 1, pict.packed_size );
      if ( pict.packed_size != size ) free( packbuf );
    } else {
      fprintf(stderr, "Couldn't find palette\n" );
      status = -1;
    }
    SDL_FreeSurface( bmp );
  } else {
    fprintf(stderr, "Couldn't open BMP %s\n", argv[4] );
    status = -1;
  }

  SDL_RWclose( out );
  return status;
}

static int compress_data( unsigned char *in, unsigned long inlen, unsigned char **out, unsigned long *outlen ) {
  int rc = 0;

#ifdef HAVE_LIBZ
  unsigned long bufsize = inlen + (inlen/1000) + 13;
  unsigned char *buf = (unsigned char *)malloc( bufsize );
#endif

  *out = in;
  *outlen = inlen;

#ifdef HAVE_LIBZ
  if ( buf ) {
    rc = compress2( buf, &bufsize, in, inlen, Z_BEST_COMPRESSION );
    if ( rc != Z_OK ) free( buf );
    else {
      *out = buf;
      *outlen = bufsize;
    }
  }
#endif

  return rc;
}

