/* mktileset -- create a terrain set for Crimson Fields
   Copyright (C) 2000-2002 Jens Granseuer

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


/* create a data set for CF, containing graphics and definitions
   for terrain

   currently, all data (except the graphics) is hardcoded into
   the program

   level designers can specify custom data files to adjust the
   graphics and/or terrain characteristics to their likes
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#ifdef HAVE_LIBZ
# include <zlib.h>
#endif

#include "SDL.h"

#include "editor.h"

#define FID_SET			MakeID('T','S','E','T')

extern const struct TerrainType TModels[];

static int compress_data( unsigned char *in, unsigned long inlen, unsigned char **out, unsigned long *outlen );

struct PicHeader {
  unsigned short colors;
  unsigned short width;
  unsigned short height;
  unsigned long packed_size;
};

struct DatHeader {
  unsigned long fid;
  unsigned short terrains;
};

/* terrain classes currently used by CF (7) are:
   plains, forest, mountains, trenches, rails, shallow water, deep water

   reserved for future expansion (2): water, roads

   the final ID: trenches
   This denotes the first trenches hex in the tileset. Trenches need to
   appear in exactly the same order as in the default set if the set
   should be usable with pioneers! */
unsigned short classids[10] = { 30, 81, 117, 150, 250, 360, 362, 361, 178, 148 };


int main( int argc, char *argv[] ) {
  SDL_Surface *bmp;
  int status, i, j, k, size;
  unsigned char num, *packbuf, *upackbuf;
  SDL_RWops *out;
  struct DatHeader dhead = { FID_SET, NUM_TERRAIN_TYPES };

  if ( argc < 3 ) {
    fprintf(stderr, "Too few of arguments\n"
                    "Usage: %s <graphics.bmp> <outfile>\n",
                    argv[0] );
    exit(-1);
  }


  if ( SDL_Init(0) < 0 ) {
    fprintf(stderr, "Couldn't init SDL: %s\n", SDL_GetError());
    exit(-1);
  }
  atexit(SDL_Quit);

  out = SDL_RWFromFile( argv[argc-1], "wb" );
  if ( !out ) {
    fprintf(stderr, "Couldn't open output file %s\n", argv[argc-1] );
    exit(-1);
  }

  status = 0;

  /* write data file header */
  SDL_WriteLE32( out, dhead.fid );
  SDL_WriteLE16( out, dhead.terrains );

  for ( i = 0; i < 10; ++i ) SDL_WriteLE16( out, classids[i] );

  /* write terrain data */
  for ( i = 0; i < NUM_TERRAIN_TYPES; ++i ) {
    SDL_WriteLE16( out, TModels[i].tt_type );
    SDL_WriteLE16( out, TModels[i].tt_image );
    SDL_RWwrite( out, &TModels[i].tt_att_mod, 1, 3 );
    SDL_WriteLE32( out, TModels[i].tt_color );
  }

  /* load graphics */
  bmp = SDL_LoadBMP(argv[1]);
  if ( bmp ) {
    if ( bmp->format->BitsPerPixel == 8 ) {
      SDL_Palette *palette = bmp->format->palette;
      struct PicHeader pict = { palette->ncolors, bmp->w, bmp->h, 0 };

      size = bmp->w * bmp->h;
      upackbuf = (unsigned char *)malloc( size );

      /* copy surface lines to buffer */
      for ( i = 0; i < bmp->h; ++i ) {
        for ( j = 0; j < bmp->w; ++j ) {
          upackbuf[j + i * bmp->w] = *((Uint8 *)bmp->pixels + j + i * bmp->pitch);
        }
      }

      status = compress_data( upackbuf, size, &packbuf, &pict.packed_size );
      if ( status ) {
        fprintf( stderr, "Error compressing data\n" );
        free( upackbuf );
        SDL_RWclose( out );
        return status;
      }

      SDL_WriteLE16( out, pict.colors );
      SDL_WriteLE16( out, pict.width );
      SDL_WriteLE16( out, pict.height );
      SDL_WriteLE32( out, pict.packed_size );

      for ( i = 0; i < palette->ncolors; ++i )
        SDL_RWwrite( out, (char *)&palette->colors[i], 1, 3 );

      SDL_RWwrite( out, packbuf, 1, pict.packed_size );
      if ( pict.packed_size != size ) free( packbuf );
    } else {
      fprintf(stderr, "Couldn't find palette\n" );
      status = -1;
    }
    SDL_FreeSurface( bmp );
  } else {
    fprintf(stderr, "Couldn't open BMP %s\n", argv[1] );
    status = -1;
  }

  SDL_RWclose( out );
  return status;
}

static int compress_data( unsigned char *in, unsigned long inlen, unsigned char **out, unsigned long *outlen ) {
  int rc = 0;

#ifdef HAVE_LIBZ
  unsigned long bufsize = inlen + (inlen/1000) + 13;
  unsigned char *buf = (unsigned char *)malloc( bufsize );
#endif

  *out = in;
  *outlen = inlen;

#ifdef HAVE_LIBZ
  if ( buf ) {
    rc = compress2( buf, &bufsize, in, inlen, Z_BEST_COMPRESSION );
    if ( rc != Z_OK ) free( buf );
    else {
      *out = buf;
      *outlen = bufsize;
    }
  }
#endif

  return rc;
}

