/* mkunitset -- create a unit set for Crimson Fields
   Copyright (C) 2000-2002 Jens Granseuer

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


/* create a data set for CF, containing graphics and definitions for units

   currently, all data (except the graphics) is hardcoded into
   the program

   level designers can specify custom data files to adjust the
   graphics and/or units to their likes
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#ifdef HAVE_LIBZ
# include <zlib.h>
#endif

#include "SDL.h"

#include "editor.h"

#define NUM_SOUNDS		0

#define FID_SET			MakeID('U','S','E','T')

static int compress_data( unsigned char *in, unsigned long inlen, unsigned char **out, unsigned long *outlen );

extern const struct UnitType UModels[];

const char *set_sounds[] = { NULL };

struct PicHeader {
  unsigned short colors;
  unsigned short width;
  unsigned short height;
  unsigned long packed_size;
};

struct DatHeader {
  unsigned long fid;
  unsigned short units;
  unsigned char sounds;
};


int main( int argc, char *argv[] ) {
  SDL_Surface *bmp;
  int status, i, j, k, size;
  unsigned char num, *packbuf, *upackbuf;
  SDL_RWops *out;
  struct DatHeader dhead = { FID_SET, NUM_UNIT_TYPES, NUM_SOUNDS };

  if ( argc <= 3 ) {
    fprintf(stderr, "Too few of arguments\n"
                    "Usage: %s <graphics.bmp> [<unit1.bmp>...] <outfile>\n",
                    argv[0] );
    exit(-1);
  }


  if ( SDL_Init(0) < 0 ) {
    fprintf(stderr, "Couldn't init SDL: %s\n", SDL_GetError());
    exit(-1);
  }
  atexit(SDL_Quit);

  out = SDL_RWFromFile( argv[argc-1], "wb" );
  if ( !out ) {
    fprintf(stderr, "Couldn't open output file %s\n", argv[argc-1] );
    exit(-1);
  }

  status = 0;

  /* write data file header */
  SDL_WriteLE32( out, dhead.fid );
  SDL_WriteLE16( out, dhead.units );

  /* write sound file names */
  SDL_RWwrite( out, &dhead.sounds, 1, 1 );
  for ( i = 0; i < NUM_SOUNDS; ++i ) {
    unsigned char len = strlen( set_sounds[i] );
    SDL_RWwrite( out, &len, 1, 1 );
    SDL_RWwrite( out, set_sounds[i], 1, len );
  }

  /* write unit data */
  for ( i = 0; i < NUM_UNIT_TYPES; ++i ) {
    SDL_WriteLE16( out, UModels[i].ut_terrain );
    SDL_WriteLE16( out, UModels[i].ut_image );
    SDL_WriteLE16( out, UModels[i].ut_flags );
    SDL_RWwrite( out, &UModels[i].ut_moves, 1, 20 );
    SDL_RWwrite( out, UModels[i].ut_name, 1, 20 );
  }

  /* load graphics */
  bmp = SDL_LoadBMP(argv[1]);
  if ( bmp ) {
    if ( bmp->format->BitsPerPixel == 8 ) {
      SDL_Palette *palette = bmp->format->palette;
      struct PicHeader pict = { palette->ncolors, bmp->w, bmp->h, 0 };

      size = bmp->w * bmp->h;
      upackbuf = (unsigned char *)malloc( size );

      /* copy surface lines to buffer */
      for ( i = 0; i < bmp->h; ++i ) {
        for ( j = 0; j < bmp->w; ++j ) {
          upackbuf[j + i * bmp->w] = *((Uint8 *)bmp->pixels + j + i * bmp->pitch);
        }
      }

      status = compress_data( upackbuf, size, &packbuf, &pict.packed_size );
      if ( status ) {
        fprintf( stderr, "Error compressing data\n" );
        free( upackbuf );
        SDL_RWclose( out );
        return status;
      }

      SDL_WriteLE16( out, pict.colors );
      SDL_WriteLE16( out, pict.width );
      SDL_WriteLE16( out, pict.height );
      SDL_WriteLE32( out, pict.packed_size );

      for ( i = 0; i < palette->ncolors; ++i )
        SDL_RWwrite( out, (char *)&palette->colors[i], 1, 3 );

      SDL_RWwrite( out, packbuf, 1, pict.packed_size );
      if ( pict.packed_size != size ) free( packbuf );
    } else {
      fprintf(stderr, "Couldn't find palette\n" );
      status = -1;
    }
    SDL_FreeSurface( bmp );
  } else {
    fprintf(stderr, "Couldn't open BMP %s\n", argv[1] );
    status = -1;
  }

  /* load optional unit portraits */
  num = (unsigned char)(argc - 3);
  SDL_RWwrite( out, &num, 1, 1 );
  for ( k = 0; (k < num) && (status == 0); ++k ) {

    bmp = SDL_LoadBMP(argv[2 + k]);
    if ( bmp ) {
      if ( bmp->format->BitsPerPixel == 8 ) {
        SDL_Palette *palette = bmp->format->palette;
        struct PicHeader pict = { palette->ncolors, bmp->w, bmp->h, 0 };

        size = bmp->w * bmp->h;
        upackbuf = (unsigned char *)malloc( size );

        /* copy surface lines to buffer */
        for ( i = 0; i < bmp->h; ++i ) {
          for ( j = 0; j < bmp->w; ++j ) {
            upackbuf[j + i * bmp->w] = *((Uint8 *)bmp->pixels + j + i * bmp->pitch);
          }
        }

        status = compress_data( upackbuf, size, &packbuf, &pict.packed_size );
        if ( status ) {
          fprintf( stderr, "Error compressing data\n" );
          free( upackbuf );
          SDL_RWclose( out );
          return status;
        }

        SDL_WriteLE16( out, pict.colors );
        SDL_WriteLE16( out, pict.width );
        SDL_WriteLE16( out, pict.height );
        SDL_WriteLE32( out, pict.packed_size );

        for ( i = 0; i < palette->ncolors; ++i )
          SDL_RWwrite( out, (char *)&palette->colors[i], 1, 3 );

        SDL_RWwrite( out, packbuf, 1, pict.packed_size );
        if ( pict.packed_size != size ) free( packbuf );
      } else {
        fprintf(stderr, "Couldn't find palette\n" );
        status = -1;
        break;
      }
      SDL_FreeSurface( bmp );
    } else {
      fprintf(stderr, "Couldn't open BMP %s\n", argv[2 + k] );
      status = -1;
      break;
    }
  }

  SDL_RWclose( out );
  return status;
}

static int compress_data( unsigned char *in, unsigned long inlen, unsigned char **out, unsigned long *outlen ) {
  int rc = 0;

#ifdef HAVE_LIBZ
  unsigned long bufsize = inlen + (inlen/1000) + 13;
  unsigned char *buf = (unsigned char *)malloc( bufsize );
#endif

  *out = in;
  *outlen = inlen;

#ifdef HAVE_LIBZ
  if ( buf ) {
    rc = compress2( buf, &bufsize, in, inlen, Z_BEST_COMPRESSION );
    if ( rc != Z_OK ) free( buf );
    else {
      *out = buf;
      *outlen = bufsize;
    }
  }
#endif

  return rc;
}

